module EvalFullComb where

import Expr2
import ResMonadComb
import qualified Data.Map as M

type State = M.Map String Double

eval :: Expr -> Res State Double
eval (Var i)  = get (M.! i)
eval (Num n)  = return n
eval (Plus a b)  = do x<- eval a; y<- eval b; return $ x+ y
eval (Minus a b) = do x<- eval a; y<- eval b; return $ x- y
eval (Times a b) = do x<- eval a; y<- eval b; return $ x* y
eval (Div a b)  = do x<- eval a; y<- eval b
                     if y == 0 then fail "/0" else return $ x / y
eval (Pick a b)  = do x<- eval a; y<- eval b; join x y

{-
-- Examples:
s = M.fromList[("a", 3), ("b", 7)]
run (eval (Plus (Var "a") (Var "b"))) s

run (eval (Plus (Num 3) (Div (Var "b") (Minus (Num 3) (Var "a"))))) s

run (eval (Plus (Num 3) (Div (Var "a") (Var "b")))) s

run (eval (Div (Pick (Var "a") (Num 5)) (Pick (Var "b") (Num 0)))) s

-}
