module ResMonad(
    Res
  , run
  , get
  , join
  ) where

-- The result monad: a combination of reader monad, list monad and maybe monad.
data Res sigma alpha = Res { run :: sigma-> [Maybe alpha] }

instance Functor (Res sigma) where
  fmap f (Res g) = Res $ fmap (fmap f). g 

instance Applicative (Res sigma) where
  pure a = Res (const [Just a])
  Res f <*> Res a = Res $ \s-> do h<-f s; i<- a s; return $ h <*> i

instance Monad (Res sigma) where
  return a = Res (const [Just a])
  Res f >>= g = Res $ \s-> do ma<- f s
                              case ma of
			        Just a-> run (g a) s
			        Nothing-> return Nothing
  fail _ = Res $ const [Nothing]

-- Elementary operations:

get :: (sigma-> alpha)-> Res sigma alpha
get f = Res $ \s-> [Just $ f s]

join :: alpha-> alpha-> Res sigma alpha
join a b = Res $ \s-> [Just a, Just b]
